#
# Seals call-type analyses in Time,  Paper Supplement
#
# Paper Title: ACOUSTIC ECOLOGY OF ANTARCTIC PINNIPEDS 
# Authors: Ilse Van Opzeeland, Sofie Van Parijs, 
#          Horst Bornemann1, Stephan Frickenhaus, 
#          Lars Kindermann1, Holger Klinck, Joachim Pltz, Olaf Boebel
#
# Marine Ecology Progress Series
#    submitted April 2010
#
#    This code is for R (see r-project.org). 
#    It depends on the R-package "ade4" and the 
#       data-file "TimeFull-kaal.txt"
#

S=read.table("TimeFull-kaal.txt",header=T,sep="\t")

# correct some NA entries
is=is.na(S[,])
S[is]=0

S$File=as.character(S$File)

# to compute weeks in the year
dm=c(31,28,31,30,31,30,31,31,30,31,30,31)
dm.s=dm; dm.s[1]=1
for (i in 2:12) dm.s[i]=dm.s[i-1]+dm[i]
S$Month=substr(S$File,5,6)
S$Day=substr(S$File,7,8)
S$YDay=dm.s[as.numeric(S$Month)]+as.numeric(S$Day)-1
S$Month=as.factor(S$Month)
S$Week=round(S$YDay/7)

# set year-month YYMM
S$YM=substr(S$File,3,6)

#set sets of seal call-types
Weddel=4:17
Leopard=18:24
Crabeater=25
Ross=26:30

#
# sum counts over specific call-types
#
S$Leo=apply(S[,Leopard],1,sum)
S$Ross=apply(S[,Ross],1,sum)
S$Weddel=apply(S[,Weddel],1,sum)
S$Crabeater=as.numeric(S[,Crabeater])

#
# time-courses of call-type sums
# unpublished figure
x11();
par(mfrow=c(4,1),mar=c(3,2,1,1))
plot(S$Leo,type="l",ylim=c(0,40),main="Leopard")
plot(S$Ross,type="l",col=2,main="Ross")
plot(S$Weddel,type="l",col=4,main="Weddel")
plot(S$Crabeater,type="l",col=3,main="Crabeater")


#########################################################
# Time-structure (hourly) over the day, month-wise
#########################################################
# max 24 hours per day, over 12 months

Hourly=function(set,nrm=TRUE)
{
 hr.inf=matrix(0,12,24)
 gg=1
 SH=vector("list",length(levels(as.factor(S$YM))) )
 # YM LOOP. over several lines!!!
 for (m in levels(as.factor(S$YM)))
 {
 S.H=S[S$YM==m,]
 S.H$hour=0
 S.H$minute=as.character(S.H$minute)
 for (i in 1:dim(S.H)[1])
 {
  if (nchar(S.H$minute[i])<3)   S.H$hour[i]=0    
  if (nchar(S.H$minute[i])==3)  S.H$hour[i]=as.numeric(substr(S.H$minute[i],1,1)) 
  if (nchar(S.H$minute[i])==4)  S.H$hour[i]=as.numeric(substr(S.H$minute[i],1,2)) 
 }
 # S.H$hour
 S.H1=matrix(0,length(levels(as.factor(S.H$hour))),length(set))
 dim(S.H1)
 hr=c()
 for ( i in 1:length(levels(as.factor(S.H$hour))) )
 { g=levels(as.factor(S.H$hour))[i]
  S.H1[i,]= apply (S.H[S.H$hour==g,set],2,sum)  
  hr[i]=g
 }
 for (i in 1:dim(S.H1)[1])
 { sm=sum(S.H1[i,])
  if (sm>0) S.H1[i,]= S.H1[i,]/sm
 }
 colnames(S.H1)=names(S[,set])
 rownames(S.H1)=hr
 rng=as.numeric(hr)/23
 cls=rgb(rng,rng,rng)

 Inform=c()
 for (i in 1:length(hr))
 {
  pi=S.H1[i,]
  lg=log(pi)
  lg[which(lg==-Inf)]=0
  Inform[i]=-sum(pi*lg)
 }
 hr.inf[gg,as.numeric(hr)+1]=Inform
 rownames(S.H1)=paste(rownames(S.H1),m,sep="h")
 if (gg==1) S.H.all=S.H1 else S.H.all=rbind(S.H.all,S.H1)
 SH[[gg]]=S.H1
 gg=gg+1
 }
 list(S.H.all=S.H.all,S.H=SH)
} 
#######################
# END Hourly function
#######################

#
# do the hourly analyses for each of the seals
#
# ATTENTION: execute only one of the three alternatives:
#  default: Wedell-analysis
S.H.W=Hourly(Weddel)

S.H.W=Hourly(Leopard) # needed for supplement figure
S.H.W=Hourly(Ross)    # needed for supplement figure



sm= apply(S.H.W$S.H.all,1,sum)
S.H.all=S.H.W$S.H.all[sm>0, ]

# to analyse month May for Wedell
S.May=S.H.W$S.H[[5]]

##################################################
# the package ade4 is needed for the COA analyses
##################################################
require(ade4)

#######################################################
# The day-time structure of May for Weddell seals
#######################################################
#  unpublished figure for day-time structure
#   for the Wedell-dataset in May only
#
dc=dudi.coa(data.frame(S.May),scannf=FALSE,nf=2)
par(mfrow = c(1,2),mar=c(1,1,1,1))
s.label(dc$co, clab = 1.4, lab = colnames(S.May))
title("a",adj=0)
s.label(dc$li, clab = 0.9,lab=substr(rownames(S.May),1,3))
title("b",adj=0)

#
# loop over all months
#
# unpublished figures
#
for (m in 1:length(S.H.W$S.H))
{
S.M=S.H.W$S.H[[m]]
if (sum(S.M)>3)  # to exclude a bad data month for Ross, having only two calls
{
 dc=dudi.coa(data.frame(S.M),scannf=FALSE,nf=2)
 x11(width=12,height=6)
 par(mfrow = c(1,2),mar=c(1,1,1,1))
 s.label(dc$co, clab = 1.2, lab = colnames(S.M))
 title("a",adj=0)
 s.label(dc$li, clab = 1)
 title("b",adj=0)
}
}

############################################################
# Week data COA function, producing the COA-graphics when called below
############################################################
Week.COA=function(set,nrm=TRUE)
{
 SW=matrix(0,35,length(set))
for ( i in 1:length(levels(as.factor(S$Week))) )
{ g=levels(as.factor(S$Week))[i]
  SW[i,]= apply (S[S$Week==g,set],2,sum)  
}
if (nrm) {
 for (i in 1:dim(SW)[1])
 { sm=sum(SW[i,])
   if (sm>0)   SW[i,]= SW[i,]/sm
 }
}
sm=apply(SW,1,sum)
colnames(SW)=names(S[,set])
rownames(SW)=levels(as.factor(S$Week))
SW1=SW[sm>0,] # filter null rows
mtch=match(rownames(SW1),as.numeric(S$Week))
mnth=paste(S$Week[mtch],S$YM[mtch])
coa.S=dudi.coa(SW1,scannf=F,nf=2)
par(mfrow = c(1,2),mar=c(1,1,1,1))
s.label(coa.S$co, clab = 0.9, lab = colnames(SW1))
title("a",adj=0)
s.label(coa.S$li, clab = 0.9,lab=mnth)
title("b",adj=0)
list(SW=SW1,coa=coa.S)
}
###########################################
# end of function for weekly analysis
###########################################

###################################
# COA for Weddell-seal call types
###################################
# Figure 4 a and b
#    saved in a PNG-file, added ellipses manually (see text)
#
x11()
png(file="fig4ab.png",width=1024,height=512)
COA=Week.COA(Weddel,nrm=FALSE)
dev.off()

######################################################################
# a heat-map to visualize week structure of activity / distributions
#######################################################################
# unpublished figure
#  requires myheatmap.R
#  the method is adapted from 
#  Gescher, C., Metfies, K., Frickenhaus, S., Knefelkamp, B., Wiltshire, K.H., 
#   Medlin, L.K.(2008). Assessing Community Composition of Prasinophytes 
#   at the Helgoland Roads Sampling Site with a DNA Microarray: 
#   A Feasibility Study. 
#   Applied and Environmental Microbiology., doi:10.1128/AEM.01271-08 .
#
source("myheatmap.R")
x11()
rng=as.numeric(rownames(COA$SW))/52
cls=rgb(rng,rng,rng)
my.heatmap(COA$SW,scale="none",RowSideColors=cls,cexRow=1.75,cexCol=1.75)

########################################
# discriminating by Weddell-seals Period
########################################
#
# Figure 5 a and b
#

W.period=c("Mo","Mo","Mo",rep("0",20),"PL","PL","PL","LM","LM","LM","Ma","Ma")
COA=Week.COA(Weddel,nrm=TRUE)
coad=discrimin.coa(data.frame(COA$SW),as.factor(W.period),scan=F)
plot(coad)

# same plot into a PNG-file,  published figure is generated from this
png(file="fig5ab.png",height=800,width=800)
plot(coad,cex=5,scale=3)
dev.off()

################################
# end of supplement material
################################ 6.4.2010 [sf]